## Generated by R/Makefile and R/make-make-pages; do not edit by hand


## contributions from NAMESPACE.init
import(stats)
import(graphics)
import(grDevices)
import(utils)
import(methods)
import(Matrix)
import(sp)
import(splines)

# Export all names
# exportPattern(".")


## contributions from NAMESPACE.ours
# *** NAMESPACE contributions from our old system
# The functions to be exported are defined in the R-code with
# lines as:
# ## Export: func1
# ## Export: func1 func2
# ## Export: method1!class
# etc...
export(inla.agaussian)
export(inla.binary.install)
export(inla.cgeneric.define)
export(inla.cgeneric.q)
export(inla.changelog)
export(inla.collect.results)
export(inla.compare.results)
export(inla.coxph)
export(inla.rbind.data.frames)
export(inla.cpo)
export(inla.cut)
export(inla.debug.graph)
export(inla.dev.new)
export(inla.doc)
export(f)
export(inla.fgn)
export(inla.geobugs2inla)
export(inla.graph2matrix)
export(inla.spy)
export(inla.group)
export(inla.hyperpar)
export(inla.idx)
export(inla.iidkd.sample)
export(inla)
export(inla.get.inlaEnv)
S3method(summary, inla.jmarginal)
S3method(print, summary.inla.jmarginal)
S3method(print, inla.jmarginal)
export(inla.rjmarginal)
export(inla.rjmarginal.eval)
export(inla.tjmarginal)
export(inla.1djmarginal)
export(inla.jp.define)
export(inla.lattice2node.mapping)
export(inla.node2lattice.mapping)
export(inla.lattice2node)
export(inla.node2lattice)
export(inla.matrix2vector)
export(inla.vector2matrix)
export(inla.make.lincomb)
export(inla.make.lincombs)
export(inla.link.log)
export(inla.link.invlog)
export(inla.link.neglog)
export(inla.link.invneglog)
export(inla.link.logit)
export(inla.link.invlogit)
export(inla.link.probit)
export(inla.link.invprobit)
export(inla.link.cloglog)
export(inla.link.invcloglog)
export(inla.link.loglog)
export(inla.link.invloglog)
export(inla.link.tan)
export(inla.link.invtan)
export(inla.link.identity)
export(inla.link.invidentity)
export(inla.link.invalid)
export(inla.link.cauchit)
export(inla.link.invcauchit)
export(inla.link.inverse)
export(inla.link.invinverse)
export(inla.link.robit)
export(inla.link.invrobit)
export(inla.link.sn)
export(inla.link.invsn)
export(inla.list.models)
export(inla.load)
export(inla.dmarginal)
export(inla.pmarginal)
export(inla.qmarginal)
export(inla.rmarginal)
export(inla.hpdmarginal)
export(inla.smarginal)
export(inla.emarginal)
export(inla.tmarginal)
export(inla.mmarginal)
export(inla.zmarginal)
export(inla.mdata)
export(is.inla.mdata)
export(as.inla.mdata)
S3method(print, inla.mdata)
S3method(merge, inla)
export(inla.merge)
export(inla.mesh.components)
export(inla.mesh.assessment)
export(inla.models)
export(inla.nmix.lambda.fitted)
export(inla.setOption)
export(inla.getOption)
export(inla.pardiso)
export(inla.pardiso.check)
export(inla.pc.ralphaw)
export(inla.pc.dalphaw)
export(inla.pc.qalphaw)
export(inla.pc.palphaw)
export(inla.pc.ar.rpacf)
export(inla.pc.ar.dpacf)
export(inla.pc.rcor0)
export(inla.pc.dcor0)
export(inla.pc.qcor0)
export(inla.pc.pcor0)
export(inla.pc.rcor1)
export(inla.pc.dcor1)
export(inla.pc.qcor1)
export(inla.pc.pcor1)
export(inla.pc.cormat.dim2p)
export(inla.pc.cormat.p2dim)
export(inla.pc.cormat.theta2R)
export(inla.pc.cormat.R2theta)
export(inla.pc.cormat.r2R)
export(inla.pc.cormat.R2r)
export(inla.pc.cormat.r2theta)
export(inla.pc.cormat.theta2r)
export(inla.pc.cormat.permute)
export(inla.pc.cormat.rtheta)
export(inla.pc.cormat.dtheta)
export(inla.pc.rgamma)
export(inla.pc.dgamma)
export(inla.pc.qgamma)
export(inla.pc.pgamma)
export(inla.pc.rgammacount)
export(inla.pc.dgammacount)
export(inla.pc.qgammacount)
export(inla.pc.pgammacount)
export(inla.pc.rgevtail)
export(inla.pc.dgevtail)
export(inla.pc.qgevtail)
export(inla.pc.pgevtail)
export(inla.pc.multvar.h.default)
export(inla.pc.multvar.simplex.d)
export(inla.pc.multvar.simplex.r)
export(inla.pc.multvar.sphere.d)
export(inla.pc.multvar.sphere.r)
export(inla.pc.rprec)
export(inla.pc.dprec)
export(inla.pc.qprec)
export(inla.pc.pprec)
export(inla.pc.rsn)
export(inla.pc.dsn)
export(inla.pc.qsn)
export(inla.pc.psn)
export(inla.pc.ddof)
S3method(plot, inla)
export(inla.posterior.sample)
export(inla.posterior.sample.eval)
S3method(print, inla)
export(inla.priors.used)
export(inla.prune)
export(inla.qstat)
export(inla.qdel)
export(inla.qget)
export(inla.qnuke)
export(inla.qlog)
S3method(print, inla.q)
S3method(summary, inla.q)
export(inla.qinv)
export(inla.qreordering)
export(inla.qsample)
export(inla.qsolve)
export(inla.read.graph)
export(inla.write.graph)
S3method(summary, inla.graph)
S3method(plot, inla.graph)
S3method(print, inla.graph.summary)
export(inla.ssh.copy.id)
export(inla.remote)
export(inla.reorderings)
export(inla.rerun)
export(inla.ks.plot)
export(inla.rgeneric.ar1.model)
export(inla.rgeneric.ar1.model.opt)
export(inla.rgeneric.iid.model)
export(inla.rgeneric.define)
export(inla.rgeneric.wrapper)
export(inla.rgeneric.q)
export(inla.hyperpar.sample)
export(inla.scale.model)
export(inla.set.control.lincomb.default)
export(inla.set.control.update.default)
export(inla.set.control.group.default)
export(inla.set.control.mix.default)
export(inla.set.control.pom.default)
export(inla.set.control.link.default)
export(inla.set.control.expert.default)
export(inla.set.control.compute.default)
export(inla.set.control.family.default)
export(inla.set.control.fixed.default)
export(inla.set.control.inla.default)
export(inla.set.control.predictor.default)
export(inla.set.control.mode.default)
export(inla.set.control.hazard.default)
export(inla.set.control.bgev.default)
export(inla.set.control.lp.scale.default)
export(inla.set.control.pardiso.default)
export(control.lincomb)
export(control.update)
export(control.group)
export(control.mix)
export(control.pom)
export(control.link)
export(control.expert)
export(control.compute)
export(control.family)
export(control.fixed)
export(control.inla)
export(control.vb)
export(control.predictor)
export(control.mode)
export(control.hazard)
export(control.bgev)
export(control.lp.scale)
export(control.pardiso)
export(control.gcpo)
export(inla.as.sparse)
export(inla.as.dgTMatrix)
S3method(summary, inla)
S3method(print, summary.inla)
export(inla.surv)
export(is.inla.surv)
export(as.inla.surv)
S3method(plot, inla.surv)
S3method(print, inla.surv)
export(inla.upgrade)
export(inla.update)
export(inla.version)


## contributions from NAMESPACE.roxygen
# Generated by roxygen2: do not edit by hand

S3method(as.inla.mesh.segment,Line)
S3method(as.inla.mesh.segment,Lines)
S3method(as.inla.mesh.segment,Polygon)
S3method(as.inla.mesh.segment,Polygons)
S3method(as.inla.mesh.segment,SpatialLines)
S3method(as.inla.mesh.segment,SpatialLinesDataFrame)
S3method(as.inla.mesh.segment,SpatialPoints)
S3method(as.inla.mesh.segment,SpatialPointsDataFrame)
S3method(as.inla.mesh.segment,SpatialPolygons)
S3method(as.inla.mesh.segment,SpatialPolygonsDataFrame)
S3method(extract.groups,inla.mesh.segment)
S3method(inla.diameter,default)
S3method(inla.diameter,inla.mesh)
S3method(inla.diameter,inla.mesh.1d)
S3method(inla.diameter,inla.mesh.lattice)
S3method(inla.diameter,inla.mesh.segment)
S3method(inla.extract.el,data.frame)
S3method(inla.extract.el,list)
S3method(inla.extract.el,matrix)
S3method(inla.mesh.project,inla.mesh)
S3method(inla.mesh.project,inla.mesh.1d)
S3method(inla.mesh.project,inla.mesh.projector)
S3method(inla.mesh.projector,inla.mesh)
S3method(inla.mesh.projector,inla.mesh.1d)
S3method(inla.mesh.segment,default)
S3method(inla.mesh.segment,inla.mesh.segment)
S3method(inla.spTransform,SpatialPoints)
S3method(inla.spTransform,SpatialPointsDataFrame)
S3method(inla.spTransform,default)
S3method(inla.spTransform,inla.mesh)
S3method(inla.spTransform,inla.mesh.lattice)
S3method(inla.spTransform,inla.mesh.segment)
S3method(inla.spde.precision,inla.spde1)
S3method(inla.spde.precision,inla.spde2)
S3method(inla.spde.result,inla.spde1)
S3method(inla.spde.result,inla.spde2)
S3method(inla.spde.sample,default)
S3method(inla.spde.sample,inla.spde)
S3method(lines,inla.mesh.segment)
S3method(plot,CRS)
S3method(plot,inla.CRS)
S3method(plot,inla.mesh)
S3method(plot,inla.trimesh)
S3method(print,summary.inla.mesh)
S3method(summary,inla.mesh)
export(as.inla.mesh.segment)
export(extract.groups)
export(inla.CRS)
export(inla.CRSargs)
export(inla.ar.pacf2acf)
export(inla.ar.pacf2phi)
export(inla.ar.phi2acf)
export(inla.ar.phi2pacf)
export(inla.as.CRS.list)
export(inla.as.CRSargs.list)
export(inla.as.list.CRS)
export(inla.as.list.CRSargs)
export(inla.as.wkt.wkt_tree)
export(inla.as.wkt_tree.wkt)
export(inla.barrier.fem)
export(inla.barrier.pcmatern)
export(inla.barrier.polygon)
export(inla.barrier.q)
export(inla.contour.segment)
export(inla.crs_get_ellipsoid_radius)
export(inla.crs_get_lengthunit)
export(inla.crs_get_wkt)
export(inla.crs_is_geocent)
export(inla.crs_set_ellipsoid_radius)
export(inla.crs_set_lengthunit)
export(inla.dBind)
export(inla.delaunay)
export(inla.diameter)
export(inla.extract.el)
export(inla.fmesher.smorg)
export(inla.generate.colors)
export(inla.has_PROJ6)
export(inla.identical.CRS)
export(inla.knmodels)
export(inla.knmodels.sample)
export(inla.matern.cov)
export(inla.matern.cov.s2)
export(inla.matern.cov.s2.unstable)
export(inla.mesh.1d)
export(inla.mesh.1d.A)
export(inla.mesh.1d.bary)
export(inla.mesh.1d.fem)
export(inla.mesh.2d)
export(inla.mesh.assessment)
export(inla.mesh.basis)
export(inla.mesh.boundary)
export(inla.mesh.components)
export(inla.mesh.create)
export(inla.mesh.create.helper)
export(inla.mesh.deriv)
export(inla.mesh.fem)
export(inla.mesh.interior)
export(inla.mesh.lattice)
export(inla.mesh.map)
export(inla.mesh.map.lim)
export(inla.mesh.project)
export(inla.mesh.projector)
export(inla.mesh.query)
export(inla.mesh.segment)
export(inla.nonconvex.hull)
export(inla.nonconvex.hull.basic)
export(inla.over_sp_mesh)
export(inla.row.kron)
export(inla.simplify.curve)
export(inla.sp2segment)
export(inla.spTransform)
export(inla.sp_get_crs)
export(inla.spde.make.A)
export(inla.spde.make.block.A)
export(inla.spde.make.index)
export(inla.spde.models)
export(inla.spde.precision)
export(inla.spde.result)
export(inla.spde.sample)
export(inla.spde1.create)
export(inla.spde1.imatern)
export(inla.spde1.matern)
export(inla.spde1.matern.osc)
export(inla.spde1.models)
export(inla.spde1.precision)
export(inla.spde1.result)
export(inla.spde2.generic)
export(inla.spde2.matern)
export(inla.spde2.matern.sd.basis)
export(inla.spde2.models)
export(inla.spde2.pcmatern)
export(inla.spde2.precision)
export(inla.spde2.result)
export(inla.spde2.theta2phi0)
export(inla.spde2.theta2phi1)
export(inla.spde2.theta2phi2)
export(inla.stack)
export(inla.stack.A)
export(inla.stack.LHS)
export(inla.stack.RHS)
export(inla.stack.compress)
export(inla.stack.data)
export(inla.stack.index)
export(inla.stack.join)
export(inla.stack.remove.unused)
export(inla.stack.sum)
export(inla.wkt_get_ellipsoid_radius)
export(inla.wkt_get_lengthunit)
export(inla.wkt_is_geocent)
export(inla.wkt_predef)
export(inla.wkt_set_ellipsoid_radius)
export(inla.wkt_set_lengthunit)
export(inla.wkt_tree_get_item)
export(inla.wkt_tree_set_item)
export(inla.wkt_unit_params)
export(meshbuilder)
export(param2.matern.orig)
exportClasses(inla)
importFrom(methods,setOldClass)
