#' inla estimation object class
#'
#' The inla class is defined in the INLA package
#' @name inla-class
#' @seealso [inla][INLA::inla]
#' @importFrom methods setOldClass
#' @exportClass inla
setOldClass("inla")
