% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Oral}
\alias{Oral}
\title{~~ data name/kind ... ~~}
\format{
A data frame with 544 observations on the following 3 variables.
\describe{ \item{region}{a numeric vector} \item{E}{a
numeric vector} \item{Y}{a numeric vector} }
}
\description{
~~ A concise (1-5 lines) description of the dataset. ~~
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}
