% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export-class.R
\name{inla-class}
\alias{inla-class}
\title{inla estimation object class}
\description{
The inla class is defined in the INLA package
}
\seealso{
\link[=inla]{inla}
}
