% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Cancer}
\alias{Cancer}
\title{~~ data name/kind ... ~~}
\format{
A data frame with 6690 observations on the following 4 variables.
\describe{
\item{Y}{Number of cases}
\item{N}{a numeric vector}
\item{Age}{a numeric vector}
\item{region}{a numeric vector}
}
}
\description{
~~ A concise (1-5 lines) description of the dataset. ~~
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}
